package de.intevation.flys.artifacts.states;

import java.text.NumberFormat;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.List;

import org.apache.log4j.Logger;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import de.intevation.artifacts.Artifact;
import de.intevation.artifacts.ArtifactNamespaceContext;
import de.intevation.artifacts.CallContext;
import de.intevation.artifacts.CallMeta;

import de.intevation.artifacts.common.utils.XMLUtils;
import de.intevation.artifacts.common.utils.XMLUtils.ElementCreator;

import de.intevation.artifactdatabase.ProtocolUtils;

import de.intevation.artifactdatabase.data.DefaultStateData;
import de.intevation.artifactdatabase.data.StateData;

import de.intevation.artifactdatabase.state.AbstractState;
import de.intevation.artifactdatabase.state.Facet;

import de.intevation.flys.artifacts.FLYSArtifact;

import de.intevation.flys.artifacts.resources.Resources;


/**
 * @author <a href="mailto:ingo.weinzierl@intevation.de">Ingo Weinzierl</a>
 */
public abstract class DefaultState extends AbstractState {

    /** The logger that is used in this class. */
    private static Logger logger = Logger.getLogger(DefaultState.class);

    public static enum ComputeType {
        FEED, ADVANCE, INIT
    }

    protected StateData getData(FLYSArtifact artifact,  String name) {
        return artifact.getData(name);
    }


    /**
     * Append to a node and return xml description relevant for gui.
     */
    public Element describeStatic(
        Artifact    artifact,
        Document    document,
        Node        root,
        CallContext context,
        String      uuid)
    {
        ElementCreator creator = new ElementCreator(
            document,
            ArtifactNamespaceContext.NAMESPACE_URI,
            ArtifactNamespaceContext.NAMESPACE_PREFIX);

        CallMeta meta = context.getMeta();

        String label = Resources.getMsg(meta, getID(), getID());
        Element ui   = ProtocolUtils.createArtNode(
            creator, "state",
            new String[] { "name", "uiprovider", "label" },
            new String[] { getID(), getUIProvider(), label });

        Map<String, StateData> theData = getData();
        if (theData == null) {
            return ui;
        }

        Iterator<String> iter = theData.keySet().iterator();
        FLYSArtifact     flys = (FLYSArtifact) artifact;

        while (iter.hasNext()) {
            String name = iter.next();
            appendStaticData(flys, context, creator, ui, name);
        }

        return ui;
    }


    protected void appendStaticData(
        FLYSArtifact   flys,
        CallContext    context,
        ElementCreator cr,
        Element        ui,
        String         name
    ) {
        StateData data  = getData(flys, name);
        String    value = (data != null) ? (String) data.getValue() : null;

        if (value == null) {
            return;
        }

        logger.debug("Append element '" + name + "' (" + value + ")");

        Element e = createStaticData(cr, context, name, value, data.getType());

        ui.appendChild(e);

    }


    /**
     * Creates a <i>data</i> element used in the static part of the DESCRIBE
     * document.
     *
     * @param creator The ElementCreator that is used to build new Elements.
     * @param meta The CallMeta object used for i18n.
     * @param name The name of the data item.
     * @param value The value as string.
     *
     * @return an Element.
     */
    protected Element createStaticData(
        ElementCreator creator,
        CallContext    cc,
        String         name,
        String         value,
        String         type
    ) {
        CallMeta meta = cc.getMeta();

        Element dataElement = creator.create("data");
        creator.addAttr(dataElement, "name", name, true);
        creator.addAttr(dataElement, "type", type, true);

        Element itemElement = creator.create("item");
        creator.addAttr(itemElement, "value", value, true);

        String attrValue = "";
        try {
            // XXX A better way to format the output would be to use the
            // 'type' value of the data objects.
            double doubleVal = Double.valueOf(value);
            Locale         l = Resources.getLocale(meta);
            NumberFormat  nf = NumberFormat.getInstance(l);

            attrValue = nf.format(doubleVal);
        }
        catch (NumberFormatException nfe) {
            attrValue = Resources.getMsg(meta, value, value);
        }

        creator.addAttr(itemElement, "label", attrValue, true);
        dataElement.appendChild(itemElement);

        return dataElement;
    }


    public Element describe(
        Artifact    artifact,
        Document    document,
        Node        root,
        CallContext context,
        String      uuid)
    {
        ElementCreator creator = new ElementCreator(
            document,
            ArtifactNamespaceContext.NAMESPACE_URI,
            ArtifactNamespaceContext.NAMESPACE_PREFIX);

        Element ui        = null;
        String uiprovider = getUIProvider();
        if (uiprovider != null) {
            ui = ProtocolUtils.createArtNode(
                creator, "dynamic",
                new String[] { "uiprovider" },
                new String[] { uiprovider });
        }
        else {
            ui = ProtocolUtils.createArtNode(creator, "dynamic", null, null);
        }

        Map<String, StateData> theData = getData();
        if (theData == null) {
            return ui;
        }

        Iterator<String> iter = theData.keySet().iterator();
        FLYSArtifact     flys = (FLYSArtifact) artifact;

        while (iter.hasNext()) {
            String    name = iter.next();
            StateData data = getData(flys, name);

            data = data != null ? data : getData(name);

            Element select = createData(creator, artifact, data, context);

            String defValue = (String) data.getValue();
            String defDesc  = null;

            if (defValue != null && defValue.length() > 0) {
                defDesc = Resources.getMsg(
                    context.getMeta(),
                    defValue,
                    defValue);
            }

            if (defValue != null && defDesc != null) {
                creator.addAttr(select, "defaultValue", defValue, true);
                creator.addAttr(select, "defaultLabel", defDesc, true);
            }

            Element choices = ProtocolUtils.createArtNode(
            creator, "choices", null, null);

            select.appendChild(choices);
            ui.appendChild(select);

            Element[] items = createItems(creator, artifact, name, context);
            if (items != null) {
                for (Element item: items) {
                    choices.appendChild(item);
                }
            }
        }

        return ui;
    }


    /**
     * This method creates the root node that contains the list of selectable
     * items.
     *
     * @param cr The ElementCreator.
     * @param name The name of the amount of data.
     *
     * @return the root node of the item list.
     */
    protected Element createData(
        ElementCreator cr,
        Artifact    artifact,
        StateData   data,
        CallContext context)
    {
        Element select = ProtocolUtils.createArtNode(
            cr, "select", null, null);
        cr.addAttr(select, "name", data.getName(), true);

        Element label = ProtocolUtils.createArtNode(
            cr, "label", null, null);

        select.appendChild(label);

        label.setTextContent(Resources.getMsg(
            context.getMeta(),
            getID(),
            getID()));

        return select;
    }


    /**
     * This method creates a list of items. These items represent the amount of
     * input data that is possible for this state.
     *
     * @param cr The ElementCreator.
     * @param name The name of the amount of data.
     *
     * @return a list of items.
     */
    protected Element[] createItems(
        ElementCreator cr,
        Artifact    artifact,
        String      name,
        CallContext context
    ) {
        return null;
    }


    /**
     * This method is used to create an <i>item</i> Element that contains two
     * further elements <i>label</i> and <i>value</i>. The label and value
     * elements both have text nodes.
     *
     * @param cr The ElementCreator used to build new Elements.
     * @param obj This implementation awaits a String array with [0] = label and
     * [1] = value.
     *
     * @return an Element.
     */
    protected Element createItem(XMLUtils.ElementCreator cr, Object obj) {
        Element item  = ProtocolUtils.createArtNode(cr, "item", null, null);
        Element label = ProtocolUtils.createArtNode(cr, "label", null, null);
        Element value = ProtocolUtils.createArtNode(cr, "value", null, null);

        String[] arr = (String[]) obj;

        label.setTextContent(arr[0]);
        value.setTextContent(arr[1]);

        item.appendChild(label);
        item.appendChild(value);

        return item;
    }


    /**
     * This method transform a given value into a StateData object.
     *
     * @param flys The FLYSArtifact.
     * @param name The name of the data object.
     * @param val The value of the data object.
     *
     * @return a StateData object with <i>name</i> and <i>val</i>ue.
     */
    public StateData transform(
        FLYSArtifact flys,
        CallContext  cc,
        String       name,
        String       val
    ) {
        logger.debug("Transform data ('" + name + "','" + val + "')");
        return new DefaultStateData(name, null, null, val);
    }


    /**
     * This method validates the inserted data and returns true, if everything
     * was correct, otherwise an exception is thrown.
     *
     * @param artifact A reference to the owner artifact.
     *
     * @return true, if everything was fine.
     */
    public boolean validate(Artifact artifact)
    throws IllegalArgumentException
    {
        return true;
    }


    /**
     * Returns which UIProvider shall be used to aid user input.
     */
    protected String getUIProvider() {
        return null;
    }


    public Object computeAdvance(
        FLYSArtifact artifact,
        String       hash,
        CallContext  context,
        List<Facet>  facets,
        Object       old
    ) {
        return null;
    }


    public Object computeFeed(
        FLYSArtifact artifact,
        String       hash,
        CallContext  context,
        List<Facet>  facets,
        Object       old
    ) {
        return null;
    }


    public Object computeInit(
        FLYSArtifact artifact,
        String       hash,
        Object       context,
        CallMeta     meta,
        List<Facet>  facets)
    {
        return null;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf-8 :
